#from Unet_Pytorch import *
from Unet_LayerNorm import *
from custom_dataset import *
from utils import *
#from utils_grad_accum import *
import torch.nn as nn
import torch
import torchvision.transforms as transforms
import os
from torch.utils.data import random_split

import matplotlib.pylab as plt
import argparse

def getting_vars():
    execution = 't'     # t for train, s for sample

    ## Model Variables
    in_channels = 1     # RGB images
    num_classes = 4     # Vehicle or Not Vehicle
    num_filters = 32    # Trying out different nums

    ## Hyperparameters
    training_batch_size = 8
    training_epochs = 100
    learning_rate = 1e-3

    ## Adding final sigmoid and thresholding layers to U-Net Model
    sigmoid = 'n'
    thresholding = 'n'

    parser = argparse.ArgumentParser()

    parser.add_argument('-execution', type = str, default = execution, choices=['t', 's'], 
                        help = 't for train, or s for sampling the model')
    parser.add_argument('-in_channels', type = int, default = in_channels, 
                        help = '# of channels present in the input data')
    parser.add_argument('-num_classes', type = int, default = num_classes, 
                        help = '# of output classes from the U-net model')
    parser.add_argument('-num_filters', type = int, default = num_filters, 
                        help = '# of filters you want the first layer of the U-net model to have')
    parser.add_argument('-training_batch_size', type = int, default = training_batch_size, 
                        help = 'Batch Size during Training and Validation')
    parser.add_argument('-training_epochs', type = int, default = training_epochs, 
                        help = '# of epochs you want to train the model')
    parser.add_argument('-learning_rate', type = float, default = learning_rate, 
                        help = 'learning rate')
    parser.add_argument('-sigmoid', type = str, default = sigmoid, choices=['y', 'n'],
                        help = 'Add sigmoid to Unet or not (y/n)')
    parser.add_argument('-threshold', type = str, default = thresholding, choices=['y', 'n'],
                        help = 'Add thresholding to Unet or not (y/n)')

    args = parser.parse_args()
    return args


if __name__ == '__main__':

    device = torch.device("cuda" if torch.cuda.is_available() else "cpu")   # finding device
    print("Device: ", device)

    args = getting_vars()

    ## Visually see what the variables are set to in the Command line interface
    print(f"Execution: {args.execution}")
    print(f"In-Channels: {args.in_channels}")
    print(f"num classes: {args.num_classes}")
    print(f"num filters: {args.num_filters}")
    print(f"training batch size: {args.training_batch_size}")
    print(f"training_epochs: {args.training_epochs}")
    print(f"learning_rate: {args.learning_rate}")
    print(f"sigmoid: {args.sigmoid}")
    print(f"Threshold end: {args.threshold}")

    ### DO NOT CHANGE THE VARIABLES IN HERE. CHANGE IT IN THE getting_vars() method   

    ## Data Paths
    current_dir = os.getcwd()
    data_dir = r'data/trimmed_audio_STFT'

    model_save_dir = os.path.join(current_dir, r'models')

    ## Set if setting the model in training mode or inference mode
    execution = args.execution  

    ## Model Variables
    in_channels = args.in_channels     # RGB images
    num_classes = args.num_classes     # Vehicle or Not Vehicle
    num_filters = args.num_filters    # Trying out different nums

    ## Hyperparameters
    training_batch_size = args.training_batch_size
    training_epochs = args.training_epochs
    learning_rate = args.learning_rate

    ## Adding final sigmoid layer to U-Net Model
    sigmoid = True if args.sigmoid == 'y' else False
    threshold = True if args.threshold == 'y' else False

    ## Data Prep
    full_dataset = CustomDataset(data_dir)

    ## Define split sizes
    train_size = int(0.8 * len(full_dataset))
    val_size = len(full_dataset) - train_size

    ## Splitting the data into training and validation CustomDatasets
    training_data, validation_data = random_split(full_dataset, [train_size, val_size])

    ## Creating DataLoaders for train and validation data
    train_dataloader = DataLoader(training_data, batch_size = training_batch_size, shuffle = True)
    val_dataloader = DataLoader(validation_data, batch_size = training_batch_size, shuffle = False)

    ## Model Creation
    model = Unet(in_channels = in_channels, n_class = num_classes, n_filters = num_filters, 
                 add_sigmoid = sigmoid).to(device)

    ## Loss Fn and Optimizer
    loss_fn = nn.MSELoss() #nn.BCEWithLogitsLoss()
    optimizer = torch.optim.Adam(model.parameters(), lr = learning_rate)
    
    ## Troubleshooting NaN loss
    #initial_loss_check(train_dataloader, model, loss_fn, device)

    if execution == 't':
        #train_loop(train_dataloader, model, loss_fn, optimizer, training_epochs, threshold, val_dataloader, model_save_dir, device)
        train_loop(train_dataloader, model, loss_fn, optimizer, training_epochs, 
                   do_threshold = threshold, val_data = val_dataloader, save_dir = model_save_dir, device = device)
   
   
   ## Still under construction LOL
    if execution == 's':
        last_train = training_epochs-1
        model.load_state_dict(torch.load(os.path.join(model_save_dir, f"model_{last_train}.pth")), map_location=device)
        model.eval() 
        print("Loaded in audio Model")
