# Assuming input_data is your preprocessed input as a numpy array
import torch
import numpy as np
import librosa
from Unet_LayerNorm import *
from custom_dataset import *
import matplotlib.pyplot as plt
import soundfile as sf

## Model Variables
root_dir = 'data/trimmed__TEST_NP_audio'
dataset = CustomDataset(root_dir)
dataloader = DataLoader(dataset, batch_size=1, shuffle=False)

predicted_audio_dir = os.path.join('PredictedAudio')
os.makedirs(predicted_audio_dir, exist_ok=True)

# Setup the model
in_channels = 1  
num_classes = 4
num_filters = 32
sigmoid = 'n'
n_fft = 512
hop_length = 512
magnitude_channel_true = None

model_save_dir = "models"
model_path = os.path.join(model_save_dir, "model_99.pth")
device = torch.device("cuda" if torch.cuda.is_available() else "cpu")

model = Unet(in_channels=in_channels, n_class=num_classes, n_filters=num_filters, add_sigmoid=sigmoid).to(device)
model.load_state_dict(torch.load(model_path, map_location=device))
model.eval()

def magnitude_to_audio(magnitude, name="test_outptut", sr=44100, n_iter=50, hop_length=512):
    # Converts an STFT magnitude array to an audio signal and saves it as a WAV file.
    # Returns:
    # - None, but saves the audio signal to 'output_audio.wav'.
    file_name = os.path.join(predicted_audio_dir, f'{name}.wav')
    magnitude = np.pad(magnitude, ((0, 1), (0, 0)), mode='constant')

    print(f"Shape of Magnitude Matrix: {magnitude.shape}")
    # Use Griffin-Lim algorithm to reconstruct the phase
    audio_signal = librosa.griffinlim(magnitude, n_iter=n_iter, n_fft=hop_length)
    np.save("test_magnitude.npy", magnitude)

    # Save the audio signal to a WAV file
    sf.write(file_name, audio_signal, sr)


# Iterate over the dataset using the DataLoader
for batch_idx, (mixture, indv_streams, norm_params) in enumerate(dataloader):
    mixture = mixture.to(device)  # Move to the same device as the model
    
    
    with torch.no_grad():
        output = model(mixture)
        if batch_idx == 5:
            magnitude_channel_true = indv_streams[0, 0, :, :]

            mean_val = norm_params["Accompaniment"]["mean"].cpu().numpy()
            std_val = norm_params["Accompaniment"]["std"].cpu().numpy()

            magnitude_to_audio(magnitude_channel_true.cpu().numpy())

            # print(mean_val)
            # print(std_val)
            

            # # Plot the magnitude_channel
            # plt.figure(figsize=(10, 4))
            # plt.imshow(magnitude_channel_true, aspect='auto', origin='lower')
            # plt.title(f'Magnitude Channel Truth')
            # plt.colorbar(format='%+2.0f dB')
            # plt.tight_layout()
            # plt.show()

            break


output_np = output.cpu().numpy()  # Move output to cpu and convert to numpy
# print(output_np.shape)  # Output shape: (1, 4, 256, 256))

reconstructed_audios = []

for i in range(output_np.shape[1]):  # Iterate over channels/classes
    # Extract the magnitude for the current channel
    magnitude_channel = output_np[0, i, :, :]
    
    # if (i == 0):
        # Plot the magnitude_channel
        # plt.figure(figsize=(10, 4))
        # plt.imshow(magnitude_channel, aspect='auto', origin='lower')
        # plt.title(f'Magnitude Channel {i}')
        # plt.colorbar(format='%+2.0f dB')
        # plt.tight_layout()
        # plt.show()
    
    # magnitude_channel = np.pad(magnitude_channel, ((0, 1), (0, 0)), mode='constant')
    
    # Invert the magnitude to an audio signal
    # audio_reconstructed = invert_stft(magnitude_channel, sr=44100, n_fft=n_fft, hop_length=hop_length, n_iter=32)
    audio_reconstructed = magnitude_to_audio(magnitude_channel, name=f'reconstructed_audio_{i}')
    # Store the reconstructed audio
    # reconstructed_audios.append(audio_reconstructed/np.max(np.abs(audio_reconstructed)))

# for i, audio in enumerate(reconstructed_audios):
#     file_name = os.path.join(predicted_audio_dir, f"reconstructed_audio_{i}.wav")
#     sf.write(file_name, audio, 44100)
