import numpy as np
import librosa

def calculate_mse(true_audio, test_audio):
    """Calculate the Mean Squared Error between two audio signals."""
    # Ensure the audio signals are of the same length
    min_length = min(len(true_audio), len(test_audio))
    true_audio = true_audio[:min_length]
    test_audio = test_audio[:min_length]
    # Calculate MSE
    mse = np.mean((true_audio - test_audio) ** 2)
    return mse

# Load the audio files
true_audio, sr_true = librosa.load('data/audio1d/true.mp3', sr=None)
layer3_audio, sr_layer3 = librosa.load('data/audio1d/3layer.wav', sr=None)
layer6_audio, sr_layer6 = librosa.load('data/audio1d/6layer.wav', sr=None)

# Calculate MSE
mse_3layer = calculate_mse(true_audio, layer3_audio)
mse_6layer = calculate_mse(true_audio, layer6_audio)

print(f"MSE between True and 3-Layer: {mse_3layer}")
print(f"MSE between True and 6-Layer: {mse_6layer}")