import os
import librosa
import numpy as np
import stempeg

def process_and_save_stft(input_dir, output_dir, n_fft=512, hop_length=512, target_size=(1024, 1024)):
    # Define stream names
    stream_names = ['Mixture', 'Drums', 'Bass', 'Accompaniment', 'Vocals']

    for name in stream_names:
        os.makedirs(os.path.join(output_dir, name), exist_ok=True)
    
    file_counter = 0
    
    for file_name in os.listdir(input_dir):
        file_path = os.path.join(input_dir, file_name)
 
        streams, sr = stempeg.read_stems(file_path, always_3d=True)
        
        for i, name in enumerate(stream_names):
            # Convert to mono
            mono_audio = np.mean(streams[i], axis=-1)  # Average channels
            # Compute STFT
            stft = librosa.stft(mono_audio, n_fft=n_fft, hop_length=hop_length)
            # Convert to magnitude
            magnitude = np.abs(stft)
            
            print("Natural shape of the magnitude:",magnitude.shape)
            trimmed_magnitude = magnitude[:target_size[0], :target_size[1]]
            print(trimmed_magnitude.shape)

            # Verify if the trimmed magnitude has the desired shape, otherwise skip saving
            if trimmed_magnitude.shape == target_size and not np.isnan(trimmed_magnitude).any():
                # Define save path with numerical naming
                save_path = os.path.join(output_dir, name, f'{file_counter}_{name}.npy')
                
                # Save STFT magnitude
                np.save(save_path, trimmed_magnitude)
                print(f"Processed and saved STFTs for file number {file_counter}, stream: {name}")
            elif trimmed_magnitude.shape != target_size:
                print(f"Skipped file number {file_counter}, stream: {name} due to mismatched size.")
            else:
                print(f"Found NaN values in STFT for file number {file_counter}, stream: {name}")
        
        file_counter += 1  # Increment the file counter after processing each file


input_dir = "C:/Users/andre/EE367/audio_separation_unet_drew/data/trimmed__TEST_audio"
output_dir = "C:/Users/andre/EE367/audio_separation_unet_drew/data/trimmed__TEST_NP_audio"
process_and_save_stft(input_dir, output_dir,target_size=(256, 256))
