import ffmpeg
import os

def split_and_trim_stem_files(source_dir, target_dir, output_duration):

    if not os.path.exists(target_dir):
        os.makedirs(target_dir)

    for filename in os.listdir(source_dir):
        if filename.endswith(".mp4"):
            source_filepath = os.path.join(source_dir, filename)
            
            probe = ffmpeg.probe(source_filepath)
            duration = float(probe['format']['duration'])

            for segment in range(int(duration // output_duration)):
                start_time = segment * output_duration
                end_time = start_time + output_duration

                if end_time <= duration:
                    target_filepath = os.path.join(target_dir, f"{filename.rsplit('.', 1)[0]}_part{segment+1}.mp4")
                    
                    ffmpeg.input(source_filepath, ss=start_time, t=output_duration) \
                          .output(target_filepath, map='0', c='copy') \
                          .run(overwrite_output=True)

                    print(f"Segment {segment+1} saved: {target_filepath}")

if __name__ == "__main__":

    source_directory = "C:/Users/andre/EE367/audio_separation_unet_drew/data/test_audio"
    target_directory = "C:/Users/andre/EE367/audio_separation_unet_drew/data/trimmed__TEST_audio"
    output_duration = float(3)
    
    split_and_trim_stem_files(source_directory, target_directory, output_duration)
